@@ -11,7 +11,7 @@
 #include <glade/glade.h>
 #include <X11/Xatom.h>
 
-#include "theme-common.h"
+#include "gnome-theme-info.h"
 #include "wm-common.h"
 #include "capplet-util.h"
 #include "eggcellrendererkeys.h"
@@ -21,6 +21,12 @@
 #define KEY_THEME_KEY "/desktop/gnome/interface/gtk_key_theme"
 #define MAX_ELEMENTS_BEFORE_SCROLLING 8
 
+#if defined(__powerpc__) && defined (__linux__)
+#define USE_FBLEVEL
+#else
+#undef USE_FBLEVEL
+#endif
+
 typedef enum {
   ALWAYS_VISIBLE,
   N_WORKSPACES_GT
@@ -35,10 +41,32 @@ typedef struct
 
 const KeyListEntry desktop_key_list[] =
 {
-  { "/apps/panel/global/run_key", ALWAYS_VISIBLE, 0 },
-  { "/apps/panel/global/menu_key", ALWAYS_VISIBLE, 0 },
-  { "/apps/panel/global/screenshot_key", ALWAYS_VISIBLE, 0 },
-  { "/apps/panel/global/window_screenshot_key", ALWAYS_VISIBLE, 0 },
+  { "/apps/metacity/global_keybindings/panel_run_dialog", ALWAYS_VISIBLE, 0 },
+  { "/apps/metacity/global_keybindings/panel_main_menu", ALWAYS_VISIBLE, 0 },
+  { "/apps/metacity/global_keybindings/run_command_screenshot", ALWAYS_VISIBLE, 0 },
+  { "/apps/metacity/global_keybindings/run_command_window_screenshot", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/volume_mute", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/volume_down", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/volume_up", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/power", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/eject", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/home", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/search", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/email", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/sleep", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/screensaver", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/help", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/www", ALWAYS_VISIBLE, 0 },
+#ifdef USE_FBLEVEL
+  { "/apps/gnome_settings_daemon/keybindings/brightness_down", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/brightness_up", ALWAYS_VISIBLE, 0 },
+#endif
+  /* Other ones that need keysyms bindings */
+  { "/apps/gnome_settings_daemon/keybindings/play", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/pause", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/stop", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/previous", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/next", ALWAYS_VISIBLE, 0 },
   { NULL }
 };
 
@@ -59,6 +87,8 @@ const KeyListEntry metacity_key_list[] =
   { "/apps/metacity/window_keybindings/raise_or_lower",            ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/raise",                     ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/lower",                     ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/maximize_vertically",       ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/maximize_horizontally",     ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/move_to_workspace_1",       N_WORKSPACES_GT, 1 },
   { "/apps/metacity/window_keybindings/move_to_workspace_2",       N_WORKSPACES_GT, 1 },
   { "/apps/metacity/window_keybindings/move_to_workspace_3",       N_WORKSPACES_GT, 2 },
@@ -110,13 +140,36 @@ typedef struct
 {
   char *gconf_key;
   guint keyval;
-  GdkModifierType mask;
+  guint keycode;
+  EggVirtualModifierType mask;
   gboolean editable;
   GtkTreeModel *model;
   guint gconf_cnxn;
+  char *description;
 } KeyEntry;
 
-static void reload_key_entries (gpointer wm_name, GladeXML *dialog);
+static void  reload_key_entries (gpointer                wm_name,
+                                 GladeXML               *dialog);
+static char* binding_name       (guint                   keyval,
+				 guint			 keycode,
+                                 EggVirtualModifierType  mask,
+                                 gboolean                translate);
+
+static GtkTreeModel*
+get_real_model (GtkTreeView *tree_view)
+{
+  GtkTreeModel *model;
+  GtkTreeModel *submodel;
+  
+  model = gtk_tree_view_get_model (tree_view);
+
+  if (model)
+    submodel = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
+  else
+    submodel = NULL;
+  
+  return submodel;
+}
 
 static void
 menu_item_activate (GtkWidget *menu_item,
@@ -146,7 +199,7 @@ make_key_theme_menu_item (const gchar *key_theme)
 
   if (!strcmp (key_theme, "Default"))
     {
-      retval = gtk_menu_item_new_with_label ("GNOME Default");
+      retval = gtk_menu_item_new_with_label (_("GNOME Default"));
       g_object_set_data_full (G_OBJECT (retval), LABEL_DATA, g_strdup ("Default"), g_free);
     }
   else
@@ -171,32 +224,35 @@ create_dialog (void)
 }
 
 static char*
-binding_name (guint            keyval,
-              GdkModifierType  mask,
-              gboolean         translate)
+binding_name (guint                   keyval,
+	      guint		      keycode,
+              EggVirtualModifierType  mask,
+              gboolean                translate)
 {
-  if (keyval != 0)
-    return gtk_accelerator_name (keyval, mask);
+  if (keyval != 0 || keycode != 0)
+    return egg_virtual_accelerator_name (keyval, keycode, mask);
   else
     return translate ? g_strdup (_("Disabled")) : g_strdup ("disabled");
 }
 
 static gboolean
-binding_from_string (const char      *str,
-                     guint           *accelerator_key,
-                     GdkModifierType *accelerator_mods)
+binding_from_string (const char             *str,
+                     guint                  *accelerator_key,
+		     guint		    *keycode,
+                     EggVirtualModifierType *accelerator_mods)
 {
   g_return_val_if_fail (accelerator_key != NULL, FALSE);
   
   if (str == NULL || (str && strcmp (str, "disabled") == 0))
     {
       *accelerator_key = 0;
+      *keycode = 0;
       *accelerator_mods = 0;
       return TRUE;
     }
 
-  gtk_accelerator_parse (str, accelerator_key, accelerator_mods);
-
+  egg_accelerator_parse_virtual (str, accelerator_key, keycode, accelerator_mods);
+  
   if (*accelerator_key == 0)
     return FALSE;
   else
@@ -226,6 +282,7 @@ accel_set_func (GtkTreeViewColumn *tree_column,
 		  "editable", FALSE,
 		  "accel_key", key_entry->keyval,
 		  "accel_mask", key_entry->mask,
+		  "keycode", key_entry->keycode,
 		  "style", PANGO_STYLE_ITALIC,
 		  NULL);
   else
@@ -234,6 +291,7 @@ accel_set_func (GtkTreeViewColumn *tree_column,
 		  "editable", TRUE,
 		  "accel_key", key_entry->keyval,
 		  "accel_mask", key_entry->mask,
+		  "keycode", key_entry->keycode,
 		  "style", PANGO_STYLE_NORMAL,
 		  NULL);
 }
@@ -272,26 +330,82 @@ keybinding_key_changed (GConfClient *client,
   key_entry = (KeyEntry *)user_data;
   key_value = gconf_value_get_string (entry->value);
 
-  binding_from_string (key_value, &key_entry->keyval, &key_entry->mask);
+  binding_from_string (key_value, &key_entry->keyval, &key_entry->keycode, &key_entry->mask);
   key_entry->editable = gconf_entry_get_is_writable (entry);
 
   /* update the model */
   gtk_tree_model_foreach (key_entry->model, keybinding_key_changed_foreach, key_entry);
 }
 
+static int
+keyentry_sort_func (GtkTreeModel *model,
+                    GtkTreeIter  *a,
+                    GtkTreeIter  *b,
+                    gpointer      user_data)
+{
+  KeyEntry *key_entry_a;
+  KeyEntry *key_entry_b;
+  char *name_a;
+  char *name_b;
+  int retval;
+
+  /* mmmmm, super-slow. */
+
+  key_entry_a = NULL;
+  gtk_tree_model_get (model, a,
+                      KEYENTRY_COLUMN, &key_entry_a,
+                      -1);  
+
+  key_entry_b = NULL;
+  gtk_tree_model_get (model, b,
+                      KEYENTRY_COLUMN, &key_entry_b,
+                      -1);
+
+  
+  if (key_entry_a != NULL)
+    name_a = binding_name (key_entry_a->keyval,
+		    	   key_entry_a->keycode,
+                           key_entry_a->mask,
+                           TRUE);
+  else
+    name_a = NULL;
+
+  if (key_entry_b != NULL)
+    name_b = binding_name (key_entry_b->keyval,
+		    	   key_entry_b->keycode,
+                           key_entry_b->mask,
+                           TRUE);
+  else
+    name_b = NULL;
+
+  if (name_a && name_b)
+    retval = g_utf8_collate (name_a, name_b);
+  else if (name_a)
+    retval = 1;
+  else if (name_b)
+    retval = -1;
+  else
+    retval = 0;
+
+  g_free (name_a);
+  g_free (name_b);
+
+  return retval;
+}
 
 static void
 clear_old_model (GladeXML  *dialog,
 		 GtkWidget *tree_view)
 {
   GtkTreeModel *model;
+  GtkTreeModel *sort_model;
   GtkTreeIter iter;
   KeyEntry *key_entry;
   gboolean valid;
   GConfClient *client;
 
   client = gconf_client_get_default ();
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
+  model = get_real_model (GTK_TREE_VIEW (tree_view));
 
   if (model != NULL)
     {
@@ -308,14 +422,28 @@ clear_old_model (GladeXML  *dialog,
 	    {
 	      gconf_client_notify_remove (client, key_entry->gconf_cnxn);
 	      g_free (key_entry->gconf_key);
+              g_free (key_entry->description);
 	      g_free (key_entry);
-	    }
+            }
 	}
       g_object_unref (model);
     }
 
   model = (GtkTreeModel *) gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER);
-  gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), model);
+  
+  sort_model = gtk_tree_model_sort_new_with_model (model);
+
+  /* N_COLUMNS is just a place to stick the extra sort function */
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (sort_model),
+                                   N_COLUMNS,
+                                   keyentry_sort_func,
+                                   NULL, NULL);
+  
+  gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), sort_model);
+
+  g_object_unref (G_OBJECT (model));
+  g_object_unref (G_OBJECT (sort_model));
+  
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WID ("actions_swindow")),
 				  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
   gtk_widget_set_usize (WID ("actions_swindow"), -1, -1);
@@ -363,7 +491,7 @@ append_keys_to_tree (GladeXML           *dialog,
   gint i, j;
 
   client = gconf_client_get_default ();
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (WID ("shortcut_treeview")));
+  model = get_real_model (GTK_TREE_VIEW (WID ("shortcut_treeview")));
 
   i = 0;
   gtk_tree_model_foreach (model, count_rows_foreach, &i);
@@ -423,15 +551,16 @@ append_keys_to_tree (GladeXML           *dialog,
 						       key_string,
 						       (GConfClientNotifyFunc) &keybinding_key_changed,
 						       key_entry, NULL, NULL);
-      binding_from_string (key_value, &key_entry->keyval, &key_entry->mask);
+      binding_from_string (key_value, &key_entry->keyval, &key_entry->keycode, &key_entry->mask);
       g_free (key_value);
+      key_entry->description = g_strdup (gconf_schema_get_short_desc (schema));
 
       if (i == MAX_ELEMENTS_BEFORE_SCROLLING)
 	{
 	  GtkRequisition rectangle;
 	  gtk_widget_ensure_style (WID ("shortcut_treeview"));
 	  gtk_widget_size_request (WID ("shortcut_treeview"), &rectangle);
-	  gtk_widget_set_size_request (WID ("actions_swindow"), -1, rectangle.height);
+	  gtk_widget_set_size_request (WID ("shortcut_treeview"), -1, rectangle.height);
 	  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WID ("actions_swindow")),
 					  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	}
@@ -440,7 +569,7 @@ append_keys_to_tree (GladeXML           *dialog,
       if (gconf_schema_get_short_desc (schema))
 	gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
 			    DESCRIPTION_COLUMN,
-                            gconf_schema_get_short_desc (schema),
+                            key_entry->description,
 			    KEYENTRY_COLUMN, key_entry,
 			    -1);
       else
@@ -539,22 +668,26 @@ cb_check_for_uniqueness (GtkTreeModel *model,
 
   if (tmp_key_entry != NULL &&
       key_entry->keyval == tmp_key_entry->keyval &&
-      key_entry->mask   == tmp_key_entry->mask)
+      key_entry->mask   == tmp_key_entry->mask &&
+      key_entry->keycode == tmp_key_entry->keycode &&
+      /* be sure we don't claim a key is a dup of itself */
+      strcmp (key_entry->gconf_key, tmp_key_entry->gconf_key) != 0)
     {
       key_entry->editable = FALSE;
       key_entry->gconf_key = tmp_key_entry->gconf_key;
+      key_entry->description = tmp_key_entry->description;
       return TRUE;
     }
   return FALSE;
 }
 
 static void
-accel_edited_callback (GtkCellRendererText *cell,
-                       const char          *path_string,
-                       guint                keyval,
-                       GdkModifierType      mask,
-		       guint                keycode,
-                       gpointer             data)
+accel_edited_callback (GtkCellRendererText   *cell,
+                       const char            *path_string,
+                       guint                  keyval,
+                       EggVirtualModifierType mask,
+		       guint		      keycode,
+                       gpointer               data)
 {
   GtkTreeView *view = (GtkTreeView *)data;
   GtkTreeModel *model;
@@ -564,7 +697,7 @@ accel_edited_callback (GtkCellRendererText *cell,
   GError *err = NULL;
   char *str;
 
-  model = gtk_tree_view_get_model (view);
+  model = get_real_model (view);
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_model_get (model, &iter,
 		      KEYENTRY_COLUMN, &key_entry,
@@ -579,29 +712,44 @@ accel_edited_callback (GtkCellRendererText *cell,
 
   tmp_key.model  = model;
   tmp_key.keyval = keyval;
+  tmp_key.keycode = keycode;
   tmp_key.mask   = mask;
+  tmp_key.gconf_key = key_entry->gconf_key;
+  tmp_key.description = NULL;
   tmp_key.editable = TRUE; /* kludge to stuff in a return flag */
-  gtk_tree_model_foreach (model, cb_check_for_uniqueness, &tmp_key);
+
+  if (keyval != 0 || keycode != 0) /* any number of keys can be disabled */
+    gtk_tree_model_foreach (model, cb_check_for_uniqueness, &tmp_key);
 
   /* flag to see if the new accelerator was in use by something */
   if (!tmp_key.editable)
     {
-      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
-				       GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
-				       GTK_MESSAGE_WARNING,
-				       GTK_BUTTONS_OK,
-				       _("That accelerator key is already in use by: %s\n"),
-				       key_entry->gconf_key);
+      GtkWidget *dialog;
+      char *name;
+
+      name = egg_virtual_accelerator_name (keyval, keycode, mask);
+      
+      dialog =
+        gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
+                                GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+                                GTK_MESSAGE_WARNING,
+                                GTK_BUTTONS_OK,
+                                _("The shortcut \"%s\" is already used for:\n \"%s\"\n"),
+                                name,
+                                tmp_key.description ?
+                                tmp_key.description : tmp_key.gconf_key);
+      g_free (name);
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
 
+      /* set it back to its previous value. */
       egg_cell_renderer_keys_set_accelerator (EGG_CELL_RENDERER_KEYS (cell),
-					      key_entry->keyval, key_entry->mask);
+					      key_entry->keyval, key_entry->keycode, key_entry->mask);
       gtk_tree_path_free (path);
       return;
     }
 
-  str = binding_name (keyval, mask, FALSE);
+  str = binding_name (keyval, keycode, mask, FALSE);
 
   gconf_client_set_string (gconf_client_get_default(),
                            key_entry->gconf_key,
@@ -629,6 +777,54 @@ accel_edited_callback (GtkCellRendererText *cell,
   gtk_tree_path_free (path);
 }
 
+static void
+accel_cleared_callback (GtkCellRendererText *cell,
+			const char          *path_string,
+			gpointer             data)
+{
+  GtkTreeView *view = (GtkTreeView *) data;
+  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
+  KeyEntry *key_entry;
+  GtkTreeIter iter;
+  GError *err = NULL;
+  GtkTreeModel *model;
+
+  model = get_real_model (view);
+  gtk_tree_model_get_iter (model, &iter, path);
+  gtk_tree_model_get (model, &iter,
+		      KEYENTRY_COLUMN, &key_entry,
+		      -1);
+
+  /* sanity check */
+  if (key_entry == NULL)
+    {
+      gtk_tree_path_free (path);
+      return;
+    }
+
+  /* Unset the key */
+  gconf_client_set_string (gconf_client_get_default(),
+			   key_entry->gconf_key,
+			   "disabled",
+			   &err);
+  if (err != NULL)
+    {
+      GtkWidget *dialog;
+
+      dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
+				       GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+				       GTK_MESSAGE_WARNING,
+				       GTK_BUTTONS_OK,
+				       _("Error unsetting accelerator in configuration database: %s\n"),
+				       err->message);
+      gtk_dialog_run (GTK_DIALOG (dialog));
+
+      gtk_widget_destroy (dialog);
+      g_error_free (err);
+      key_entry->editable = FALSE;
+    }
+  
+}
 
 static void
 theme_changed_func (gpointer  uri,
@@ -643,13 +839,13 @@ theme_changed_func (gpointer  uri,
   GList *list;
 
   client = gconf_client_get_default ();
-  key_theme_list = theme_common_get_list ();
+  key_theme_list = gnome_theme_info_find_by_type (GNOME_THEME_GTK_2_KEYBINDING);
 
   omenu = WID ("key_theme_omenu");
   menu = gtk_menu_new ();
   for (list = key_theme_list; list; list = list->next)
     {
-      ThemeInfo *info = list->data;
+      GnomeThemeInfo *info = list->data;
 
       if (! info->has_keybinding)
 	continue;
@@ -732,7 +928,7 @@ cb_dialog_response (GtkWidget *widget, gint response_id, gpointer data)
 {
 	if (response_id == GTK_RESPONSE_HELP)
 		capplet_help (GTK_WINDOW (widget),
-			      "wgoscustdesk.xml",
+			      "user-guide.xml",
 			      "goscustdesk-39");
 	else
 		gtk_main_quit ();
@@ -751,11 +947,11 @@ setup_dialog (GladeXML *dialog)
 
   client = gconf_client_get_default ();
 
-  key_theme_list = theme_common_get_list ();
+  key_theme_list = gnome_theme_info_find_by_type (GNOME_THEME_GTK_2_KEYBINDING);
 
   for (list = key_theme_list; list; list = list->next)
     {
-      ThemeInfo *info = list->data;
+      GnomeThemeInfo *info = list->data;
       if (info->has_keybinding)
 	{
 	  found_keys = TRUE;
@@ -777,7 +973,7 @@ setup_dialog (GladeXML *dialog)
   else
     {
       theme_changed_func (NULL, dialog);
-      theme_common_register_theme_change ((GFunc) theme_changed_func, dialog);
+      gnome_theme_info_register_theme_change ((GFunc) theme_changed_func, dialog);
       gconf_client_add_dir (client, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
       gconf_client_notify_add (client,
 			       KEY_THEME_KEY,
@@ -794,26 +990,34 @@ setup_dialog (GladeXML *dialog)
 						     gtk_cell_renderer_text_new (),
 						     "text", DESCRIPTION_COLUMN,
 						     NULL);
-  gtk_tree_view_column_set_resizable (column, TRUE);
+  gtk_tree_view_column_set_resizable (column, FALSE);
 
   gtk_tree_view_append_column (GTK_TREE_VIEW (WID ("shortcut_treeview")), column);
-
+  gtk_tree_view_column_set_sort_column_id (column, DESCRIPTION_COLUMN);  
+  
   renderer = (GtkCellRenderer *) g_object_new (EGG_TYPE_CELL_RENDERER_KEYS,
 					       "editable", TRUE,
 					       "accel_mode", EGG_CELL_RENDERER_KEYS_MODE_X,
 					       NULL);
 
   g_signal_connect (G_OBJECT (renderer),
-		    "keys_edited",
+		    "accel_edited",
                     G_CALLBACK (accel_edited_callback),
                     WID ("shortcut_treeview"));
 
+  g_signal_connect (G_OBJECT (renderer),
+		    "accel_cleared",
+                    G_CALLBACK (accel_cleared_callback),
+                    WID ("shortcut_treeview"));
+
   column = gtk_tree_view_column_new_with_attributes (_("Shortcut"), renderer, NULL);
   gtk_tree_view_column_set_cell_data_func (column, renderer, accel_set_func, NULL, NULL);
-  gtk_tree_view_column_set_resizable (column, TRUE);
+  gtk_tree_view_column_set_resizable (column, FALSE);
 
   gtk_tree_view_append_column (GTK_TREE_VIEW (WID ("shortcut_treeview")), column);
-
+  /* N_COLUMNS is just a place to stick the extra sort function */
+  gtk_tree_view_column_set_sort_column_id (column, N_COLUMNS); 
+  
   gconf_client_add_dir (client, "/apps/gnome_keybinding_properties", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_add_dir (client, "/apps/metacity/general", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_notify_add (client,
@@ -847,6 +1051,8 @@ main (int argc, char *argv[])
 		      GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 		      NULL);
 
+  gnome_theme_init (NULL);
+
   activate_settings_daemon ();
 
   dialog = create_dialog ();
