@@ -1,4 +1,4 @@
-/* -*- mode: c; style: linux -*- */
+/* vim: set sw=8: -*- mode: c; style: linux -*- */
 
 /* capplet-dir.c
  * Copyright (C) 2000, 2001 Ximian, Inc.
@@ -54,44 +54,23 @@ CappletDirView *(*get_view_cb) (CappletDir *dir, CappletDirView *launcher);
 /* nice global table for capplet lookup */
 GHashTable *capplet_hash = NULL;
 
-static char * 
-find_icon (const char *icon, GnomeDesktopItem *dentry) 
+static GdkPixbuf * 
+find_icon (GnomeDesktopItem *dentry) 
 {
-        char *icon_file = NULL;
+	GdkPixbuf *res;
+	char const *icon;
+	GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
 
-	if (icon && icon[0]) {
-		icon_file = g_strdup (icon);
-	}
-	
-	if (icon_file) {
-		if (icon_file[0] != '/')
-		{
-			gchar *old = icon_file;
-			icon_file = g_build_filename (GNOMECC_ICONS_DIR, old, NULL);
-			g_free (old);
-		}
-		if (!g_file_test (icon_file, G_FILE_TEST_EXISTS) || g_file_test(icon_file, G_FILE_TEST_IS_DIR))
-		{
-			const gchar *icon;
-			g_free (icon_file);
-			icon = gnome_desktop_item_get_string (dentry, GNOME_DESKTOP_ITEM_ICON);
-			if (icon)
-				icon_file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, icon, TRUE, NULL);
-	
-			if (!icon_file)
-				icon_file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-unknown.png", TRUE, NULL);
-		}
-	} else {
-		icon_file = gnome_program_locate_file
-			(gnome_program_get (), GNOME_FILE_DOMAIN_APP_PIXMAP,
-			 "control-center2.png", TRUE, NULL);
-	}
+	icon = gnome_desktop_item_get_string (dentry, GNOME_DESKTOP_ITEM_ICON);
 
-	if (!icon_file) { /* if icon_file still NULL */
-		icon_file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-unknown.png", TRUE, NULL);
-	}
-
-	return icon_file;
+	if (icon == NULL || icon[0] == 0)
+		icon = "gnome-settings";
+	res = gtk_icon_theme_load_icon (icon_theme, icon, 48, 0, NULL);
+	if (res == NULL)
+		res = gtk_icon_theme_load_icon (icon_theme, "gnome-unknown", 48, 0, NULL);
+	if (res == NULL)
+		res = gtk_icon_theme_load_icon (icon_theme, "gtk-missing-image", 48, 0, NULL);
+	return res;
 }
 
 CappletDirEntry *
@@ -138,8 +117,7 @@ capplet_new (CappletDir *dir, gchar *desktop_path)
 
 	entry->label = g_strdup (gnome_desktop_item_get_localestring (dentry,
 			GNOME_DESKTOP_ITEM_NAME));
-	entry->icon = find_icon (gnome_desktop_item_get_string (dentry, GNOME_DESKTOP_ITEM_ICON), dentry);
-	entry->pb = gdk_pixbuf_new_from_file (entry->icon, NULL);
+	entry->icon = find_icon (dentry);
 	entry->uri = gnome_vfs_uri_new (desktop_path);
 	entry->exec = vec;
 	entry->dir = dir;
@@ -187,16 +165,7 @@ capplet_dir_new (CappletDir *dir, gchar *dir_path)
 		entry->label = g_strdup (gnome_desktop_item_get_localestring (
 				entry->entry,
 				GNOME_DESKTOP_ITEM_NAME));
-		entry->icon = find_icon (gnome_desktop_item_get_string (entry->entry,
-									GNOME_DESKTOP_ITEM_ICON),
-					 entry->entry);
-
-		if (!entry->icon)
-			entry->icon = gnome_program_locate_file
-				(gnome_program_get (), GNOME_FILE_DOMAIN_APP_PIXMAP,
-				 "control-center2.png", TRUE, NULL);
-
-		entry->pb = gdk_pixbuf_new_from_file (entry->icon, NULL);
+		entry->icon = find_icon (entry->entry);
 	} else {
 		/* If the .directory file could not be found or read, abort */
 		g_free (capplet_dir);
@@ -228,7 +197,7 @@ capplet_dir_entry_destroy (CappletDirEntry *entry)
 	}
 
 	g_free (entry->label);
-	g_free (entry->icon);
+	g_object_unref (entry->icon);
 	gnome_vfs_uri_unref (entry->uri);
 	g_strfreev (entry->exec);
 	gnome_desktop_item_unref (entry->entry);
